@@ -32,3 +32,8 @@ ERR_CONN_RESET = "Target connection reset."
 ERR_CONN_RESET_FAIL = "Target connection reset -- considered a failure case when triggered from post_send"
 
 ERR_CALLBACK_FUNC = "A custom {func_name} callback function raised an uncought error.\n"
+ERR_NAME_NO_RESOLVE = "Failed to resolve block name '{0}' in context '{1}'"
+ERR_NAME_NOT_FOUND = "Cannot find block with name '{0}'"
+ERR_NAME_TOO_MANY = (
+    "Unable to resolve block name '{0}'. Use an absolute or relative name instead." " Too many potential matches: {1}"
+)
