@@ -26,9 +26,9 @@ class String(blocks.Block):
         if not self.value:
             raise exception.SullyRuntimeError("MISSING LEGO.xdr_string DEFAULT VALUE")
 
-        self.push(primitives.String(self.value))
+        self.push(primitives.String())
 
-    def render(self):
+    def render(self, mutation_context=None):
         """
         We overload and extend the render routine in order to properly pad and prefix the string.
 
