@@ -1,36 +1,32 @@
 from .base_primitive import BasePrimitive
+from .. import helpers
 
 
 class Delim(BasePrimitive):
-    def __init__(self, value=None, fuzzable=True, name=None):
-        """
-        Represent a delimiter such as :,\r,\n, ,=,>,< etc... Mutations include repetition, substitution and exclusion.
+    r"""Represent a delimiter such as :,\r,\n, ,=,>,< etc... Mutations include repetition, substitution and exclusion.
 
-        @type  value:    chr
-        @param value:    Original value
-        @type  fuzzable: bool
-        @param fuzzable: (Optional, def=True) Enable/disable fuzzing of this primitive
-        @type  name:     str
-        @param name:     (Optional, def=None) Specifying a name gives you direct access to a primitive
-        """
+    :param name: Name, for referencing later. Names should always be provided, but if not, a default name will be given,
+        defaults to None
+    :type name: str, optional
+    :param default_value: Value used when the element is not being fuzzed - should typically represent a valid value.
+    :type default_value: char, optional
+    :param fuzzable: Enable/disable fuzzing of this primitive, defaults to true
+    :type fuzzable: bool, optional
+    """
 
-        super(Delim, self).__init__()
+    def __init__(self, name=None, default_value=" ", *args, **kwargs):
+        super(Delim, self).__init__(name=name, default_value=default_value, *args, **kwargs)
 
-        self._fuzzable = fuzzable
-        self._name = name
-        self._value = self._original_value = value
-
-        if self._value:
-            self._fuzz_library.append(self._value * 2)
-            self._fuzz_library.append(self._value * 5)
-            self._fuzz_library.append(self._value * 10)
-            self._fuzz_library.append(self._value * 25)
-            self._fuzz_library.append(self._value * 100)
-            self._fuzz_library.append(self._value * 500)
-            self._fuzz_library.append(self._value * 1000)
+        self._fuzz_library.append(self._default_value * 2)
+        self._fuzz_library.append(self._default_value * 5)
+        self._fuzz_library.append(self._default_value * 10)
+        self._fuzz_library.append(self._default_value * 25)
+        self._fuzz_library.append(self._default_value * 100)
+        self._fuzz_library.append(self._default_value * 500)
+        self._fuzz_library.append(self._default_value * 1000)
 
         self._fuzz_library.append("")
-        if self._value == " ":
+        if self._default_value == " ":
             self._fuzz_library.append("\t")
             self._fuzz_library.append("\t" * 2)
             self._fuzz_library.append("\t" * 100)
@@ -72,6 +68,7 @@ class Delim(BasePrimitive):
         self._fuzz_library.append("\r\n" * 128)
         self._fuzz_library.append("\r\n" * 512)
 
-    @property
-    def name(self):
-        return self._name
+    def encode(self, value, mutation_context):
+        if value is None:
+            value = b""
+        return helpers.str_to_bytes(value)
